// reads psf components from plain file

#include "psfex.h"

using namespace std;
using namespace CCfits;

int main(int argc, char **argv)
{
 
  if(argc!=4)
  {
        cout << "syntax: " << argv[0] << " [input psf tab] [input star tab] [output fits file]" << endl;
        return 0;
  }

  psfex p(argv[1]);
  ifstream instar(argv[2]);
  
  double buf;
  int nstars=0;
  vector<double> x,y;
  while(!instar.eof())
  {
   instar >> buf;
   x.push_back(buf);
   instar >> buf;
   y.push_back(buf);
   nstars++;
  }
  nstars--;
  cout << "drawing " << nstars << " stars" << endl;
  
  FITS *pFits = 0; 
  long naxis    =   3;      
  long naxes[3] = { 48, 48, nstars };

  try
  {          
        const std::string fileName(argv[3]);
        pFits = new FITS(fileName, DOUBLE_IMG , naxis , naxes );
  }
  catch (FITS::CantCreate)
  {
        cerr << "cannot create FITS file " << argv[3] << endl;
        return 1;
  }

  valarray <double> image(0., 48*48*nstars); // we're just going to jam our data in here ...

  double m = p.maxsize(4); // this is how far out the psf allows to be interpolated

  double result[48][48]; // buffer
	
  for(int n=0; n<nstars; n++)
  {
	double sum = 0.;

	// imagine, for a second, x and y are 100 and m is 9.5
	// then this goes from 91 to 109 in i and j

	double centx = 23.0+fmod(x[n],1.0); // this is a fancy way of centering stars in postage stamps; cf. GREAT10 star challenge
	double centy = 23.0+fmod(y[n],1.0);

	for(int j=0; j<48; j++) // y range
	for(int i=0; i<48; i++) // x range
	{
		if(fabs(j-centy)<m && fabs(i-centx)<m)
			sum += result[i][j] = p.pixel_lanczos(i-centx,j-centy,x[n]-0.5,y[n]-0.5, 4);
		else
			result[i][j]=0.;
	}

	for(int j=0; j<48; j++) // y range
	for(int i=0; i<48; i++) // x range
	{
		image[i+48*j+48*48*n] = result[i][j]/sum;
	}
	cout << '.' << flush;
  }

  cout << "\nwritten stars to memory" << endl;

  try
  {
    pFits->pHDU().write(1,48*48*nstars,image);
  }
  catch (FitsException e)
  {
     cout << e.message() << endl;
     return 1;
  }

  return 0;
}
